module Chervil
  class Repl
    def initialize
      @env = Env.new
    end

    def display(obj)
      case obj
      when true
        '#t'
      when false
        '#f'
      when AST::Function
        "<function: #{obj.name.name}(#{obj.params.map(&:name).join(' ')})>"
      when Float
        if obj.floor.to_f == obj
          obj.to_i
        else
          obj
        end
      else
        obj
      end
    end

    def run
      begin
        while line = Readline.readline("> ", true)
          break if line == "(exit)"
          lexer = Lexer.new(line)
          tree = Parser.new(lexer).parse

          if tree.is_a? Error
            puts tree
            next
          end

          results = Interpreter.new(tree, @env).interpret
          results.each { |result| puts display(result) unless result.nil? }
        end
      rescue Interrupt
        puts
        repl(@env)
      end
    end
  end
end
